@@ -224,6 +224,9 @@ class SpatialMap:
         else:
             self._sites = Sites(positions, sublattices)
 
+        if self.num_sites != data.size:
+            raise RuntimeError("Data size doesn't match number of sites")
+
     @property
     def num_sites(self) -> int:
         """Total number of lattice sites"""
@@ -234,6 +237,10 @@ class SpatialMap:
         """1D array of values for each site, i.e. maps directly to x, y, z site coordinates"""
         return self._data
 
+    @data.setter
+    def data(self, value):
+        self._data = value
+
     @property
     def positions(self) -> Positions:
         """Lattice site positions. Named tuple with x, y, z fields, each a 1D array."""
@@ -527,8 +534,8 @@ class Structure:
 
     def __getitem__(self, idx):
         """Same rules as numpy indexing"""
-        return self.__class__(self._sites[idx], self._hoppings[idx],
-                              [b[idx] for b in self._boundaries])
+        return Structure(self._sites[idx], self._hoppings[idx],
+                         [b[idx] for b in self._boundaries])
 
     def cropped(self, **limits):
         """Return a copy which retains only the sites within the given limits
